@@ -1,4 +1,3 @@
-# -*- CPERL -*-
 # /=====================================================================\ #
 # |  LaTeXML::Util::Pathname                                            | #
 # | Pathname Utilities for LaTeXML                                      | #
@@ -42,16 +41,20 @@ our @EXPORT = qw( &pathname_find &pathname_findall
   &pathname_is_absolute &pathname_is_contained
   &pathname_is_url &pathname_is_literaldata
   &pathname_protocol
-  &pathname_cwd &pathname_mkdir &pathname_copy);
+  &pathname_cwd &pathname_chdir &pathname_mkdir &pathname_copy
+  &pathname_installation);
 
 # NOTE: For absolute pathnames, the directory component starts with
 # whatever File::Spec considers to be the volume, or "/".
 #======================================================================
 # Ioan Sucan suggests switching this to '\\' for windows, but notes
 # that it works as it is, so we'll leave it (for now).
-my $SEP         = '/';                          # [CONSTANT]
-my $LITERAL_RE  = '(?:literal)(?=:)';           # [CONSTANT]
-my $PROTOCOL_RE = '(?:https|http|ftp)(?=:)';    # [CONSTANT]
+### my $SEP         = '/';                          # [CONSTANT]
+# Some indicators that this is not sufficient? (calls to libraries/externals???)
+# PRELIMINARY test, probably need to be even more careful
+my $SEP         = ($^O =~ /^(MSWin32|NetWare)$/ ? '\\' : '/');    # [CONSTANT]
+my $LITERAL_RE  = '(?:literal)(?=:)';                             # [CONSTANT]
+my $PROTOCOL_RE = '(?:https|http|ftp)(?=:)';                      # [CONSTANT]
 
 #======================================================================
 # pathname_make(dir=>dir, name=>name, type=>type);
@@ -88,6 +91,8 @@ sub pathname_split {
 
 use Carp;
 
+# This likely needs portability work!!! (particularly regarding urls, separators, ...)
+# AND, care about symbolic links and collapsing ../ !!!
 sub pathname_canonical {
   my ($pathname) = @_;
   if ($pathname =~ /^($LITERAL_RE)/) {
@@ -97,6 +102,7 @@ sub pathname_canonical {
   #  File::Spec->canonpath($pathname); }
   $pathname =~ s|^~|$ENV{HOME}|;
   # We CAN canonicalize urls, but we need to be careful about the // before host!
+  # OHHH, but we DON'T want \ for separator!
   my $urlprefix = undef;
   if ($pathname =~ s|^($PROTOCOL_RE//[^/]*)/|/|) {
     $urlprefix = $1; }
@@ -162,7 +168,7 @@ sub pathname_is_contained {
   my $rel = pathname_canonical(pathname_relative(pathname_absolute($pathname),
       pathname_absolute($base)));
   # If the relative pathname starts with "../" that it apparently is NOT underneath base!
-  return ($rel =~ m|^\.\./| ? undef : $rel); }
+  return ($rel =~ m|^\.\.(?:/\|\Q$SEP\E)| ? undef : $rel); }
 
 # pathname_relative($pathname,$base) => $relativepathname
 # If $pathname is an absolute, non-URL pathname,
@@ -191,7 +197,17 @@ sub pathname_timestamp {
   return -f $pathname ? (stat($pathname))[9] : 0; }
 
 sub pathname_cwd {
-  return pathname_canonical(cwd()); }
+  if (my $cwd = cwd()) {
+    return pathname_canonical($cwd); }
+  else {
+    Fatal('expected', 'cwd', undef,
+      "Could not determine current working directory (cwd)",
+      "Perhaps a problem with Perl's locale settings?");
+    return; } }
+
+sub pathname_chdir {
+  my ($directory) = @_;
+  return chdir($directory); }
 
 sub pathname_mkdir {
   my ($directory) = @_;
@@ -245,7 +261,11 @@ sub pathname_copy {
 #    was installed, by appending it to the paths.
 
 # This is presumably daemon safe...
-my @INSTALLDIRS = grep { -d $_ } map { pathname_canonical("$_/LaTeXML") } @INC;    # [CONSTANT]
+my @INSTALLDIRS = grep { (-f "$_.pm") && (-d $_) }
+  map { pathname_canonical($_ . $SEP . 'LaTeXML') } @INC;    # [CONSTANT]
+
+sub pathname_installation {
+  return $INSTALLDIRS[0]; }
 
 sub pathname_find {
   my ($pathname, %options) = @_;
